import matplotlib as mpl
from matplotlib import cm
import matplotlib.pylab as plt
import numpy as np
import matplotlib.colors as colors
from matplotlib import ticker
from qutip import *


def plot_wigner(rho,xlim,steps):

    fig, ax = plt.subplots()
    xvec = np.linspace(-xlim,xlim,steps)
    # Wigner function
    W = wigner(rho, xvec, xvec)

    # make sure any negativity isn't just small numerical errors:
    # if it's too small (<0.005) set to zero
    if  abs(W.min()) < 0.005:
        W[W<0]=0


    # scale 1/pi
    #wlim = abs(W).max()
    #levels = np.linspace(-1/np.pi, 1/np.pi, 256)
    #norm=mpl.colors.Normalize(-wlim, wlim)


    # adapt scaling
    low = -0.06
    high=1/np.pi
    norm=mpl.colors.Normalize(low,high)
    levels = np.linspace(low,high, 256)


    plt1 = ax.contourf(xvec, xvec, W, levels, cmap=cm.RdBu_r, norm=mpl.colors.DivergingNorm(0))
    cb1 = fig.colorbar(plt1, ax=ax, format =r'$%.2f$',norm=norm)

    # set number of colorbar ticks
    tick_locator = ticker.MaxNLocator(nbins=7)
    cb1.locator = tick_locator
    cb1.update_ticks()


    return ax


def get_WLN(rho,xlim,steps):
    xvec = np.linspace(-xlim,xlim,steps)
    W = wigner(rho, xvec, xvec)
    dx=xvec[1]-xvec[0]
    int_dx = np.trapz(np.abs(W)-W)*dx
    neg = np.trapz(int_dx)*dx
    WLN =  np.log(neg + 1)
    return WLN
